local _, c = UnitClass("player")
if c ~= "ROGUE" then return end

local modeOverride

PoisonSwapper = LibStub("AceAddon-3.0"):NewAddon("PoisonSwapper", "AceEvent-3.0", "AceBucket-3.0")
local mod = PoisonSwapper
local LDB = LibStub:GetLibrary("LibDataBroker-1.1")

-- With Rank
local deadlyName, _, deadlyIcon = GetSpellInfo(57970)

-- Without rank
local instantName, _, instant_icon 		= GetSpellInfo(8680)
local woundName, _, wound_icon	 		= GetSpellInfo(13219)
local cripplingName, _, crippling_icon 	= GetSpellInfo(25809)
local mindNumbingName, _, mnIcon	 	= GetSpellInfo(25810)
local mnCheck = mindNumbingName:gsub("-", " ")
local icons = {
	[instantName] = instant_icon,
	[woundName] = wound_icon,
	[cripplingName] = crippling_icon,
	[mnCheck] = mindNumbingName,
	[deadlyName] = deadlyIcon
}

local tt = CreateFrame("GameTooltip", "PoisonSwapTooltip", UIParent, "GameTooltipTemplate")
tt:SetOwner(UIParent, "ANCHOR_NONE")

local ldbObj = LDB:NewDataObject("PoisonSwapper", {
	type = "data source",
	icon = deadlyIcon,
	label = "PoisonSwapper",
	text  = "|cff00ff00Enabled|r",	
	OnClick = function(self, btn)
		mod:Toggle()
	end
})

local function scanFor(bag, slot, poison)
	tt:SetBagItem(bag, slot)
	poison = poison:lower()
	for i = 1, tt:NumLines() do
		local l = _G["PoisonSwapTooltipTextLeft" .. i]
		if l then
			local text = l:GetText()
			if text then
				local poisonMatch = text:lower():match(poison)
				if poisonMatch then
					return true
				end
			end
		end
	end
	return false
end

local function checkForPoison(targetName, targetStacks, timeLeft)
	local stacks, expires, name, unit
	for i = 1, 40 do 
		name, _, _, stacks, _, _, expires, unit = UnitDebuff("target", i)
		if name == targetName and unit == "player" then 					
			break
		end
		if not name then break end
	end
	if expires then
		return expires - GetTime() > timeLeft and (targetStacks == 1 or stacks >= targetStacks)
	end
end

function mod:Equip(ofType)
	local location = self:FindWeapon(ofType)
	if location then
		EquipmentManager_EquipContainerItem(EquipmentManager_EquipItemByLocation(location, 17))
	end
	ldbObj.icon = icons[ofType]
end

function mod:HasImpPoisons()
	if self.hasImpPoisons ~= nil then return self.hasImpPoisons end
	local _, _, _, _, points = GetTalentInfo(1, 11)
	self.hasImpPoisons = points >= 3
	ldbObj.icon = self.hasImpPoisons and instant_icon or wound_icon
	return self.hasImpPoisons
end

function mod:PVESwap()
	local useDamage = checkForPoison(deadlyName, 5, 5)
	if useDamage then
		if self:HasImpPoisons() then			
			self:Equip(instantName)
		else
			self:Equip(woundName)
		end
	else
		self:Equip(deadlyName)
	end
end

function mod:PVPSwap()
	local _, targetClass = UnitClass("target")
	local powerType = UnitPowerType("target")
	local useMN, useDamage
	local _, _, _, _, points = GetTalentInfo(1, 18)
	local hasDeadlyBrew = points > 0
	
	if powerType == 0 and targetClass ~= "HUNTER" then
		useMN = hasDeadlyBrew or checkForPoison(cripplingName, 1, 5)
		if useMN then
			useDamage = checkForPoison(mindNumbingName, 1, 5)
		end
	else
		useDamage = hasDeadlyBrew or checkForPoison(cripplingName, 1, 5)
	end	
	if useDamage then
		if self:HasImpPoisons() then			
			self:Equip(instantName)
		else
			self:Equip(woundName)
		end
	elseif useMN then
		self:Equip(mnCheck)
	else
		self:Equip(cripplingName)
	end
end

function mod:CheckSwap()
	local forcePVE = modeOverride == "pve"
	local pvp = (modeOverride ~= "pve") and (modeOverride == "pvp" or UnitIsPVP("target") and UnitCanAttack("player", "target"))
	if pvp then
		self:PVPSwap()
	else
		self:PVESwap()
	end
end

local buffer = {}
function mod:FindWeapon(ofType)
	wipe(buffer)
	GetInventoryItemsForSlot(17, buffer)
	for location, itemID in pairs(buffer) do
		local player, bank, bags, slot, bag = EquipmentManager_UnpackLocation(location)
		if bags and bag then
			if scanFor(bag, slot, ofType) then
				return location
			end
		end
	end
	return nil
end

function mod:HandleEvent()
	if not PoisonSwapperEnabled then return end
	if not InCombatLockdown() or not UnitCanAttack("player", "target") then return end
	self:CheckSwap()
end

function mod:OnInitialize()
	if PoisonSwapperEnabled == nil then PoisonSwapperEnabled = true end
	self:SetEnabledState(PoisonSwapperEnabled)
	self:Toggle(PoisonSwapperEnabled)
	SLASH_POISONSWAPPER1 = "/ps"
	SlashCmdList["POISONSWAPPER"] = mod.SlashCommand
end

function mod.SlashCommand(msg)
	msg = msg:lower()
	if msg == "pvp" or msg == "pve" or msg == "auto" then
		print(("PoisonSwapper is now in |cffffff00%s mode|r"):format(msg))
		modeOverride = msg
		return
	end
	
	mod:Toggle()
end

function mod:Toggle(state)
	PoisonSwapperEnabled = state == nil and (not PoisonSwapperEnabled) or state
	local color, state
	if PoisonSwapperEnabled then
		color, state = "ff00ff00", "Enabled"
	else
		color, state = "ffff0000", "Disabled"
	end
	ldbObj.text = ("|c%s%s|r"):format(color, state)
	print(("PoisonSwapper: |c%s%s|r"):format(color, state))
end

function mod:OnEnable()
	self:RegisterEvent("PLAYER_TARGET_CHANGED", "HandleEvent")
	self:RegisterBucketEvent("UNIT_AURA", 0.1, "HandleEvent")
	self:RegisterEvent("CHARACTER_POINTS_CHANGED")
end

function mod:CHARACTER_POINTS_CHANGED()
	self.hasImpPoisons = nil
end